;
;REC80 Extended Remote Control for the TI92.
;
;  by Sami Khawam (sKhawam@bigfoot.com)
;
;  http://unet.univie.ac.at/~a9501901
;
; Ported to the 92 y Antoine MERCIER (a.mercier@caramail.com)
;
;
;This software uses the IR Link as a remote control
;for controling many devices. For informations on
;how to build and how to use the IR Link see my
;home page:
;
; http://unet.univie.ac.at/~a9501901
;
;
;This software enable to control all the devices that
;uses the REC80 protocol. This protol is used by many
;manufacturers, among them:
;
; Akai, Cannon, Goldstar, Hitachi, Kenwood, NEC, Onkyo
; Pioneer, TEAC, Yamaha and many of other Japanese
; manufacturers.
;
;
;The REC80 code is formed by 3 bytes: 2 Group bytes
;and one Command byte. The complement of the Command
;byte is also tranmitted so that the other end can
;check if the transfer was right.
;
;For those codes, see the rem34bg by Bjorn Gahm.
;
;This version supports the extended REC80 protocol.
;In the extended REC80 the 2 Group bytes are not
;complement of each other and this version let
;you choose each byte.
;If you don't want the extended REC80 protocol
;see the rec80 program.
;
;The use of the programm is very easy. Only some
;keys are needed:
;
;-Up,Down:       Inc- and Derement the Command byte.
;-Left,Right:    Inc- and Derement the 1st Group byte.
;-F1, F2:        Inc- and Derement the 2nd Group byte.
;-Del:           Send the signal.
;
;
;
;
;--
; Sami Khawam (sKhawam@bigfoot.com)
;
; http://unet.univie.ac.at/~a9501901
;
	include "tios.h"
 	include "flib.h"
	include "hexlib.h"
	include "irlib.h"
	xdef	_main
	xdef	_comment

_main:
	jsr		flib::clr_scr
	move.w	#2,-(a7)
	jsr	tios::FontSetSys
	lea	2(a7),a7
	move.w	#4,-(a7)					; couleur
	pea		Text1(pc)
	move.w	#14,-(a7)					; ordonne
	move.w	#5,-(a7)					; abscisse
	jsr		tios::DrawStrXY
	lea		10(a7),a7
	move.w	#4,-(a7)					; couleur
	pea		Text2(pc)
	move.w	#30,-(a7)					; ordonne
	move.w	#5,-(a7)					; abscisse
	jsr		tios::DrawStrXY
	lea		10(a7),a7
	move.w	#4,-(a7)					; couleur
	pea		Text3(pc)
	move.w	#46,-(a7)					; ordonne
	move.w	#5,-(a7)					; abscisse
	jsr		tios::DrawStrXY
	lea		10(a7),a7
Loop:
	clr.l	d0
	move.w	Group1,d0
	move.l	#2,d1
	move.l	#11,d2
	move.l	#3,d4
	jsr		hexlib::put_hex
	move.b	Command1,d0
	not		d0
	move.b	d0,Command2
	move.w	Command1,d0
	move.l	#4,d1
	move.l	#11,d2
	move.l	#3,d4
	jsr		hexlib::put_hex
Loop1:
	jsr		flib::idle_loop
	cmp.w	#264,d0					; esc
	beq		quit
	cmp.w	#340,d0					; droite
	beq		inc_Grp
	cmp.w	#337,d0					; gauche
	beq		dec_Grp
	cmp.w	#344,d0					; bas
	beq		dec_Cmd
	cmp.w	#338,d0					; haut
	beq		inc_Cmd
	cmp.w	#257,d0					; del
	beq		SendData
	cmp.w	#268,d0					; f1
	beq		up_Grp
	cmp.w	#272,d0					; f5
	beq		down_Grp
	bra		Loop1
quit:
	jsr		tios::reset_link
	rts
dec_Grp:
	sub.b	#1,Group2
	bra		Loop
inc_Grp:
	add.b	#1,Group2
	bra		Loop
down_Grp:
	sub.b	#1,Group1
	bra		Loop
up_Grp:
	add.b	#1,Group1
	bra		Loop
dec_Cmd:
	sub.b	#1,Command
	bra		Loop
inc_Cmd:
	add.b	#1,Command
	bra		Loop
SendData:
	lea		Code(pc),a0
	move.l	#22,d0
ClearCode:
	clr.b	(a0)+
	dbra	d0,ClearCode
	bsr		Encode
	lea		Data(pc),a5
	move.l	#6,d7
 	bsr		SendBytes
	lea		Code(pc),a0
	move.l	#19,d7
 	bsr		SendBytes
	bra		Loop

SendBytes:
	move.b	(a5)+,d3
	jsr		irlib::PutByte
	dbra	d7,SendBytes
	rts

; This function encodes the data and stores it in 'Code'
Encode:              
	lea		Code(pc),a0
	lea		Group(pc),a1
	clr.l	d0
	move.l	#8,d7
	move.l	#3,d2
ENextByte:
	move.l	#7,d1
 	move.b	(a1),d0
ENextBit:
	bsr		ShiftL1
	bsr		ShiftL
	rol.b	#1,d0
	bcc		EZero
	bsr		ShiftL
	bsr		ShiftL
EZero:
	dbra	d1,ENextBit
	adda.l	#1,a1
	dbra	d2,ENextByte
	move.l	#15,d1
ENStp1
	bsr		ShiftL1
	dbra	d1,ENStp1
	move.l	#3,d1
ENStp2
	bsr		ShiftL
	dbra	d1,ENStp2
	bsr		ShiftL1
	move.b	(a0),d0
	lsl.b	d7,d0
	move.b	d0,(a0)
	rts
ShiftL:
	lsl.b	#1,d3
SLCont:
	subi.b	#1,d7
	tst.b	d7
	bne		EndSL
	move.b	d3,(a0)+
	move.l	#8,d7
EndSL:
	rts
ShiftL1:
	lsl.b	#1,d3
	ori.b	#1,d3
	bra		SLCont


Group:
Group1:
 dc.b $4b
Group2:
 dc.b $f4
Command:
Command1:
 dc.b $3a
Command2:
 dc.b 0
Data:
 dc.b 23,21,$0F,$13,$FF,$FF,0       ; Data + Header
Code:
 dc.b 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

Text1:
 dc.b "Group   : ",0
Text2:
 dc.b "Command : ",0
Text3:	dc.b	"Send    : Del",0
_comment:	dc.b	"v1.0 by S.KHAWAM",0

	end
